class DBConnector:
    def save_graph(self, local_graph):
        pass

    def get_reader_endpoint(self):
        pass

    def get_writer_endpoint(self):
        pass

    def disconnect(self):
        pass
